""":mod:`wand.assertions` --- Input assertion helpers
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

This module checks user input before calling MagickWands C-API methods.


.. versionadded:: 0.5.4
"""

import numbers
try:
    from collections.abc import Sequence
except ImportError:
    from collections import Sequence

from .compat import string_type


def assert_bool(**kwargs):
    """Ensure all given values are boolean.

    :raises TypeError: if value is not ``True`` or ``False.

    .. versionadded:: 0.5.4
    """
    for label, subject in kwargs.items():
        if not isinstance(subject, bool):
            fmt = "{0} must be a bool, not {1}"
            msg = fmt.format(label, repr(subject))
            raise TypeError(msg)


def assert_color(**kwargs):
    """Ensure all given values are instances of :class:`~wand.color.Color`.

    :raises TypeError: if value is not :class:`~wand.color.Color`.

    .. versionadded:: 0.5.4
    """
    for label, subject in kwargs.items():
        if not isinstance(subject, Color):
            fmt = "Expecting an instance of wand.color.Color for {0}, not {1}"
            msg = fmt.format(label, repr(subject))
            raise TypeError(msg)


def assert_counting_number(**kwargs):
    """Ensure all given values are natural integer.

    :raises TypeError: if value is not an integer.
    :raises ValueError: if value is less than ``1``.

    .. versionadded:: 0.5.4
    """
    assert_integer(**kwargs)
    for label, subject in kwargs.items():
        if subject < 1:
            fmt = "{0}={1} must be an natural number greater than 0"
            msg = fmt.format(label, subject)
            raise ValueError(msg)


def assert_integer(**kwargs):
    """Ensure all given values are an integer.

    :raises TypeError: if value is not an integer.

    .. versionadded:: 0.5.4
    """
    for label, subject in kwargs.items():
        if not isinstance(subject, numbers.Integral):
            fmt = "{0} must be an integer, not {1}"
            msg = fmt.format(label, repr(subject))
            raise TypeError(msg)


def assert_real(**kwargs):
    """Ensure all given values are real numbers.

    :raises TypeError: if value is not a real number.

    .. versionadded:: 0.5.4
    """
    for label, subject in kwargs.items():
        if not isinstance(subject, numbers.Real):
            fmt = "{0} must be a real number, not {1}"
            msg = fmt.format(label, repr(subject))
            raise TypeError(msg)


def assert_unsigned_integer(**kwargs):
    """Ensure all given values are positive integer.

    :raises TypeError: if value is not an integer.
    :raises ValueError: if value is less than ``0``.

    .. versionadded:: 0.5.4
    """
    assert_integer(**kwargs)
    for label, subject in kwargs.items():
        if subject < 0:
            fmt = "{0}={1} must be a positive integer"
            msg = fmt.format(label, subject)
            raise ValueError(msg)


def assert_coordinate(**kwargs):
    """Ensure all given values are a sequence of 2 real numbers.

    :raises TypeError: if value is not a pair of doubles.

    .. versionadded:: 0.6.0
    """
    for label, subject in kwargs.items():
        if not isinstance(subject, Sequence):
            fmt = "'{0}' must be a pair of real numbers, not {1}"
            msg = fmt.format(label, repr(subject))
            raise TypeError(msg)
        elif len(subject) != 2:
            fmt = "'{0}' must be a exactly 2 real numbers, not {1}"
            msg = fmt.format(label, len(subject))
            raise ValueError(msg)
        elif not isinstance(subject[0], numbers.Real):
            fmt = "first entry of '{0}' must be a real number, not {1}"
            msg = fmt.format(label, repr(subject[0]))
            raise TypeError(msg)
        elif not isinstance(subject[1], numbers.Real):
            fmt = "second entry of '{0}' must be a real number, not {1}"
            msg = fmt.format(label, repr(subject[1]))
            raise TypeError(msg)


def assert_string(**kwargs):
    for label, subject in kwargs.items():
        if not isinstance(subject, string_type):
            fmt = "{0} must be a string, not {1}"
            msg = fmt.format(label, repr(subject))
            raise TypeError(msg)


def in_list(options, label, **kwargs):
    for subject_label, subject in kwargs.items():
        if subject not in options:
            fmt = "{0} must be defined in {1}, not {2}"
            msg = fmt.format(subject_label, label, repr(subject))
            raise ValueError(msg)


def string_in_list(options, label, **kwargs):
    assert_string(**kwargs)
    in_list(options, label, **kwargs)


# Lazy load recursive import
from .color import Color  # noqa: E402



""":mod:`wand.exceptions` --- Errors and warnings
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

This module maps MagickWand API's errors and warnings to Python's native
exceptions and warnings. You can catch all MagickWand errors using Python's
natural way to catch errors.

.. seealso::

   `ImageMagick Exceptions <http://www.imagemagick.org/script/exception.php>`_

.. versionadded:: 0.1.1

.. versionchanged:: 0.5.8
   Warning & Error Exceptions are now explicitly defined. Previously
   ImageMagick domain-based errors were dynamically generated at runtime.
"""


class WandException(Exception):
    """All Wand-related exceptions are derived from this class."""


class BaseWarning(WandException, Warning):
    """Base class for Wand-related warnings.

    .. versionadded:: 0.4.4

    """


class BaseError(WandException):
    """Base class for Wand-related errors.

    .. versionadded:: 0.4.4

    """


class BaseFatalError(WandException):
    """Base class for Wand-related fatal errors.

    .. versionadded:: 0.4.4

    """


class WandLibraryVersionError(WandException):
    """Base class for Wand-related ImageMagick version errors.

    .. versionadded:: 0.3.2

    """


class WandRuntimeError(WandException, RuntimeError):
    """Generic class for Wand-related runtime errors.

    .. versionadded:: 0.5.2
    """


class ResourceLimitWarning(BaseWarning, MemoryError):
    """A program resource is exhausted e.g. not enough memory."""
    wand_error_code = 300


class ResourceLimitError(BaseError, MemoryError):
    """A program resource is exhausted e.g. not enough memory."""
    wand_error_code = 400


class ResourceLimitFatalError(BaseFatalError, MemoryError):
    """A program resource is exhausted e.g. not enough memory."""
    wand_error_code = 700


class TypeWarning(BaseWarning):
    """A font is unavailable; a substitution may have occurred."""
    wand_error_code = 305


class TypeError(BaseError):
    """A font is unavailable; a substitution may have occurred."""
    wand_error_code = 405


class TypeFatalError(BaseFatalError):
    """A font is unavailable; a substitution may have occurred."""
    wand_error_code = 705


class OptionWarning(BaseWarning):
    """A command-line option was malformed."""
    wand_error_code = 310


class OptionError(BaseError):
    """A command-line option was malformed."""
    wand_error_code = 410


class OptionFatalError(BaseFatalError):
    """A command-line option was malformed."""
    wand_error_code = 710


class DelegateWarning(BaseWarning):
    """An ImageMagick delegate failed to complete."""
    wand_error_code = 315


class DelegateError(BaseError):
    """An ImageMagick delegate failed to complete."""
    wand_error_code = 415


class DelegateFatalError(BaseFatalError):
    """An ImageMagick delegate failed to complete."""
    wand_error_code = 715


class MissingDelegateWarning(BaseWarning, ImportError):
    """The image type can not be read or written because the appropriate;
       delegate is missing."""
    wand_error_code = 320


class MissingDelegateError(BaseError, ImportError):
    """The image type can not be read or written because the appropriate;
       delegate is missing."""
    wand_error_code = 420


class MissingDelegateFatalError(BaseFatalError, ImportError):
    """The image type can not be read or written because the appropriate;
       delegate is missing."""
    wand_error_code = 720


class CorruptImageWarning(BaseWarning, ValueError):
    """The image file may be corrupt."""
    wand_error_code = 325


class CorruptImageError(BaseError, ValueError):
    """The image file may be corrupt."""
    wand_error_code = 425


class CorruptImageFatalError(BaseFatalError, ValueError):
    """The image file may be corrupt."""
    wand_error_code = 725


class FileOpenWarning(BaseWarning, IOError):
    """The image file could not be opened for reading or writing."""
    wand_error_code = 330


class FileOpenError(BaseError, IOError):
    """The image file could not be opened for reading or writing."""
    wand_error_code = 430


class FileOpenFatalError(BaseFatalError, IOError):
    """The image file could not be opened for reading or writing."""
    wand_error_code = 730


class BlobWarning(BaseWarning, IOError):
    """A binary large object could not be allocated, read, or written."""
    wand_error_code = 335


class BlobError(BaseError, IOError):
    """A binary large object could not be allocated, read, or written."""
    wand_error_code = 435


class BlobFatalError(BaseFatalError, IOError):
    """A binary large object could not be allocated, read, or written."""
    wand_error_code = 735


class StreamWarning(BaseWarning, IOError):
    """There was a problem reading or writing from a stream."""
    wand_error_code = 340


class StreamError(BaseError, IOError):
    """There was a problem reading or writing from a stream."""
    wand_error_code = 440


class StreamFatalError(BaseFatalError, IOError):
    """There was a problem reading or writing from a stream."""
    wand_error_code = 740


class CacheWarning(BaseWarning):
    """Pixels could not be read or written to the pixel cache."""
    wand_error_code = 345


class CacheError(BaseError):
    """Pixels could not be read or written to the pixel cache."""
    wand_error_code = 445


class CacheFatalError(BaseFatalError):
    """Pixels could not be read or written to the pixel cache."""
    wand_error_code = 745


class CoderWarning(BaseWarning):
    """There was a problem with an image coder."""
    wand_error_code = 350


class CoderError(BaseError):
    """There was a problem with an image coder."""
    wand_error_code = 450


class CoderFatalError(BaseFatalError):
    """There was a problem with an image coder."""
    wand_error_code = 750


class ModuleWarning(BaseWarning):
    """There was a problem with an image module."""
    wand_error_code = 355


class ModuleError(BaseError):
    """There was a problem with an image module."""
    wand_error_code = 455


class ModuleFatalError(BaseFatalError):
    """There was a problem with an image module."""
    wand_error_code = 755


class DrawWarning(BaseWarning):
    """A drawing operation failed."""
    wand_error_code = 360


class DrawError(BaseError):
    """A drawing operation failed."""
    wand_error_code = 460


class DrawFatalError(BaseFatalError):
    """A drawing operation failed."""
    wand_error_code = 760


class ImageWarning(BaseWarning):
    """The operation could not complete due to an incompatible image."""
    wand_error_code = 365


class ImageError(BaseError):
    """The operation could not complete due to an incompatible image."""
    wand_error_code = 465


class ImageFatalError(BaseFatalError):
    """The operation could not complete due to an incompatible image."""
    wand_error_code = 765


class WandWarning(BaseWarning):
    """There was a problem specific to the MagickWand API."""
    wand_error_code = 370


class WandError(BaseError):
    """There was a problem specific to the MagickWand API."""
    wand_error_code = 470


class WandFatalError(BaseFatalError):
    """There was a problem specific to the MagickWand API."""
    wand_error_code = 770


class RandomWarning(BaseWarning):
    """There is a problem generating a true or pseudo-random number."""
    wand_error_code = 375


class RandomError(BaseError):
    """There is a problem generating a true or pseudo-random number."""
    wand_error_code = 475


class RandomFatalError(BaseFatalError):
    """There is a problem generating a true or pseudo-random number."""
    wand_error_code = 775


class XServerWarning(BaseWarning):
    """An X resource is unavailable."""
    wand_error_code = 380


class XServerError(BaseError):
    """An X resource is unavailable."""
    wand_error_code = 480


class XServerFatalError(BaseFatalError):
    """An X resource is unavailable."""
    wand_error_code = 780


class MonitorWarning(BaseWarning):
    """There was a problem activating the progress monitor."""
    wand_error_code = 385


class MonitorError(BaseError):
    """There was a problem activating the progress monitor."""
    wand_error_code = 485


class MonitorFatalError(BaseFatalError):
    """There was a problem activating the progress monitor."""
    wand_error_code = 785


class RegistryWarning(BaseWarning):
    """There was a problem getting or setting the registry."""
    wand_error_code = 390


class RegistryError(BaseError):
    """There was a problem getting or setting the registry."""
    wand_error_code = 490


class RegistryFatalError(BaseFatalError):
    """There was a problem getting or setting the registry."""
    wand_error_code = 790


class ConfigureWarning(BaseWarning):
    """There was a problem getting a configuration file."""
    wand_error_code = 395


class ConfigureError(BaseError):
    """There was a problem getting a configuration file."""
    wand_error_code = 495


class ConfigureFatalError(BaseFatalError):
    """There was a problem getting a configuration file."""
    wand_error_code = 795


class PolicyWarning(BaseWarning):
    """A policy denies access to a delegate, coder, filter, path, or
       resource."""
    wand_error_code = 399


class PolicyError(BaseError):
    """A policy denies access to a delegate, coder, filter, path, or
       resource."""
    wand_error_code = 499


class PolicyFatalError(BaseFatalError):
    """A policy denies access to a delegate, coder, filter, path, or
       resource."""
    wand_error_code = 799


#: (:class:`dict`) The dictionary of (code, exc_type).
TYPE_MAP = {
    300: ResourceLimitWarning,
    305: TypeWarning,
    310: OptionWarning,
    315: DelegateWarning,
    320: MissingDelegateWarning,
    325: CorruptImageWarning,
    330: FileOpenWarning,
    335: BlobWarning,
    340: StreamWarning,
    345: CacheWarning,
    350: CoderWarning,
    355: ModuleWarning,
    360: DrawWarning,
    365: ImageWarning,
    370: WandWarning,
    375: RandomWarning,
    380: XServerWarning,
    385: MonitorWarning,
    390: RegistryWarning,
    395: ConfigureWarning,
    399: PolicyWarning,
    400: ResourceLimitError,
    405: TypeError,
    410: OptionError,
    415: DelegateError,
    420: MissingDelegateError,
    425: CorruptImageError,
    430: FileOpenError,
    435: BlobError,
    440: StreamError,
    445: CacheError,
    450: CoderError,
    455: ModuleError,
    460: DrawError,
    465: ImageError,
    470: WandError,
    475: RandomError,
    480: XServerError,
    485: MonitorError,
    490: RegistryError,
    495: ConfigureError,
    499: PolicyError,
    700: ResourceLimitFatalError,
    705: TypeFatalError,
    710: OptionFatalError,
    715: DelegateFatalError,
    720: MissingDelegateFatalError,
    725: CorruptImageFatalError,
    730: FileOpenFatalError,
    735: BlobFatalError,
    740: StreamFatalError,
    745: CacheFatalError,
    750: CoderFatalError,
    755: ModuleFatalError,
    760: DrawFatalError,
    765: ImageFatalError,
    770: WandFatalError,
    775: RandomFatalError,
    780: XServerFatalError,
    785: MonitorFatalError,
    790: RegistryFatalError,
    795: ConfigureFatalError,
    799: PolicyFatalError,
}



""":mod:`wand.cdefs.magick_property` --- Magick-Property definitions
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

.. versionadded:: 0.5.0
"""
from ctypes import (POINTER, c_void_p, c_char_p, c_size_t, c_ubyte, c_uint,
                    c_int, c_ulong, c_double, c_bool)
from wand.cdefs.wandtypes import c_ssize_t

__all__ = ('load',)


def load(lib, IM_VERSION):
    """Define Magick Wand property methods. The ImageMagick version is given as
    a second argument for comparison. This will quick to determine which
    methods are available from the library, and can be implemented as::

        if IM_VERSION < 0x700:
            # ... do ImageMagick-6 methods ...
        else
            # ... do ImageMagick-7 methods ...

    .. seealso::

        #include "wand/magick-property.h"
        // Or
        #include "MagickWand/magick-property.h"

    :param lib: the loaded ``MagickWand`` library
    :type lib: :class:`ctypes.CDLL`
    :param IM_VERSION: the ImageMagick version number (i.e. 0x0689)
    :type IM_VERSION: :class:`numbers.Integral`

    .. versionadded:: 0.5.0

    """
    lib.MagickDeleteImageArtifact.argtypes = [c_void_p, c_char_p]
    lib.MagickDeleteImageArtifact.restype = c_bool
    lib.MagickDeleteImageProperty.argtypes = [c_void_p, c_char_p]
    lib.MagickDeleteImageProperty.restype = c_bool
    lib.MagickDeleteOption.argtypes = [c_void_p, c_char_p]
    lib.MagickDeleteOption.restype = c_bool
    lib.MagickGetAntialias.argtypes = [c_void_p]
    lib.MagickGetAntialias.restype = c_bool
    lib.MagickGetBackgroundColor.argtypes = [c_void_p]
    lib.MagickGetBackgroundColor.restype = c_void_p
    lib.MagickGetColorspace.argtypes = [c_void_p]
    lib.MagickGetColorspace.restype = c_int
    lib.MagickGetCompression.argtypes = [c_void_p]
    lib.MagickGetCompression.restype = c_int
    lib.MagickGetCompressionQuality.argtypes = [c_void_p]
    lib.MagickGetCompressionQuality.restype = c_size_t
    lib.MagickGetFont.argtypes = [c_void_p]
    lib.MagickGetFont.restype = c_void_p
    lib.MagickGetGravity.argtypes = [c_void_p]
    lib.MagickGetGravity.restype = c_int
    lib.MagickGetImageArtifact.argtypes = [c_void_p, c_char_p]
    lib.MagickGetImageArtifact.restype = c_void_p
    lib.MagickGetImageArtifacts.argtypes = [
        c_void_p, c_char_p, POINTER(c_size_t)
    ]
    lib.MagickGetImageArtifacts.restype = POINTER(c_void_p)
    lib.MagickGetImageProfile.argtypes = [
        c_void_p, c_char_p, POINTER(c_size_t)
    ]
    lib.MagickGetImageProfile.restype = POINTER(c_ubyte)
    lib.MagickGetImageProfiles.argtypes = [
        c_void_p, c_char_p, POINTER(c_size_t)
    ]
    lib.MagickGetImageProfiles.restype = POINTER(c_void_p)
    lib.MagickGetImageProperty.argtypes = [c_void_p, c_char_p]
    lib.MagickGetImageProperty.restype = c_void_p
    lib.MagickGetImageProperties.argtypes = [
        c_void_p, c_char_p, POINTER(c_size_t)
    ]
    lib.MagickGetImageProperties.restype = POINTER(c_void_p)
    lib.MagickGetInterlaceScheme.argtypes = [c_void_p]
    lib.MagickGetInterlaceScheme.restype = c_int
    lib.MagickGetOption.argtypes = [c_void_p, c_char_p]
    lib.MagickGetOption.restype = c_void_p
    lib.MagickGetPointsize.argtypes = [c_void_p]
    lib.MagickGetPointsize.restype = c_double
    lib.MagickGetQuantumRange.argtypes = [POINTER(c_size_t)]
    lib.MagickGetResource.argtypes = [c_int]
    lib.MagickGetResource.restype = c_size_t
    lib.MagickGetResourceLimit.argtypes = [c_int]
    lib.MagickGetResourceLimit.restype = c_size_t
    lib.MagickGetSamplingFactors.argtypes = [c_void_p, POINTER(c_size_t)]
    lib.MagickGetSamplingFactors.restype = POINTER(c_double)
    lib.MagickGetSize.argtypes = [c_void_p, POINTER(c_uint), POINTER(c_uint)]
    lib.MagickGetSize.restype = c_int
    lib.MagickQueryConfigureOption.argtypes = [c_char_p]
    lib.MagickQueryConfigureOption.restype = c_void_p
    lib.MagickQueryConfigureOptions.argtypes = [c_char_p, POINTER(c_size_t)]
    lib.MagickQueryConfigureOptions.restype = POINTER(c_void_p)
    lib.MagickQueryFontMetrics.argtypes = [c_void_p, c_void_p, c_char_p]
    lib.MagickQueryFontMetrics.restype = POINTER(c_double)
    lib.MagickQueryFonts.argtypes = [c_char_p, POINTER(c_size_t)]
    lib.MagickQueryFonts.restype = POINTER(c_void_p)
    lib.MagickQueryFormats.argtypes = [c_char_p, POINTER(c_size_t)]
    lib.MagickQueryFormats.restype = POINTER(c_void_p)
    lib.MagickQueryMultilineFontMetrics.argtypes = [
        c_void_p, c_void_p, c_char_p
    ]
    lib.MagickQueryMultilineFontMetrics.restype = POINTER(c_double)
    lib.MagickRemoveImageProfile.argtypes = [
        c_void_p, c_char_p, POINTER(c_size_t)
    ]
    lib.MagickRemoveImageProfile.restype = POINTER(c_ubyte)
    lib.MagickSetAntialias.argtypes = [c_void_p, c_int]
    lib.MagickSetAntialias.restype = c_bool
    lib.MagickSetBackgroundColor.argtypes = [c_void_p, c_void_p]
    lib.MagickSetBackgroundColor.restype = c_bool
    lib.MagickSetColorspace.argtypes = [c_void_p, c_int]
    lib.MagickSetColorspace.restype = c_bool
    lib.MagickSetCompression.argtypes = [c_void_p, c_int]
    lib.MagickSetCompression.restype = c_bool
    lib.MagickSetCompressionQuality.argtypes = [c_void_p, c_size_t]
    lib.MagickSetCompressionQuality.restype = c_bool
    lib.MagickSetDepth.argtypes = [c_void_p, c_uint]
    lib.MagickSetDepth.restype = c_bool
    lib.MagickSetExtract.argtypes = [c_void_p, c_char_p]
    lib.MagickSetExtract.restype = c_bool
    lib.MagickSetFilename.argtypes = [c_void_p, c_char_p]
    lib.MagickSetFilename.restype = c_bool
    lib.MagickSetFont.argtypes = [c_void_p, c_char_p]
    lib.MagickSetFont.restype = c_bool
    lib.MagickSetFormat.argtypes = [c_void_p, c_char_p]
    lib.MagickSetFormat.restype = c_bool
    lib.MagickSetGravity.argtypes = [c_void_p, c_int]
    lib.MagickSetGravity.restype = c_bool
    lib.MagickSetImageArtifact.argtypes = [c_void_p, c_char_p, c_char_p]
    lib.MagickSetImageProfile.argtypes = [
        c_void_p, c_char_p, c_void_p, c_size_t
    ]
    lib.MagickSetImageProfile.restype = c_bool
    lib.MagickSetImageProperty.argtypes = [c_void_p, c_char_p, c_char_p]
    lib.MagickSetImageProperty.restype = c_bool
    lib.MagickSetInterlaceScheme.argtypes = [c_void_p, c_int]
    lib.MagickSetInterlaceScheme.restype = c_bool
    lib.MagickSetInterpolateMethod.argtypes = [c_void_p, c_int]
    lib.MagickSetInterpolateMethod.restype = c_bool
    lib.MagickSetOption.argtypes = [c_void_p, c_char_p, c_char_p]
    lib.MagickSetOption.restype = c_bool
    lib.MagickSetOrientation.argtypes = [c_void_p, c_int]
    lib.MagickSetOrientation.restype = c_bool
    lib.MagickSetPage.argtypes = [
        c_void_p, c_size_t, c_size_t, c_ssize_t, c_ssize_t
    ]
    lib.MagickSetPage.restype = c_bool
    lib.MagickSetPassphrase.argtypes = [c_void_p, c_char_p]
    lib.MagickSetPassphrase.restype = c_bool
    lib.MagickSetPointsize.argtypes = [c_void_p, c_double]
    lib.MagickSetPointsize.restype = c_bool
    lib.MagickSetResolution.argtypes = [c_void_p, c_double, c_double]
    lib.MagickSetResolution.restype = c_bool
    lib.MagickSetResourceLimit.argtypes = [c_int, c_size_t]
    lib.MagickSetResourceLimit.restype = c_bool
    lib.MagickSetSamplingFactors.argtypes = [
        c_void_p, c_size_t, POINTER(c_double)
    ]
    lib.MagickSetSamplingFactors.restype = c_bool
    if IM_VERSION >= 0x708:
        try:
            lib.MagickSetSeed.argtypes = [c_void_p, c_ulong]
        except AttributeError:
            lib.MagickSetSeed = None
    else:
        lib.MagickSetSeed = None
    try:
        lib.MagickSetSecurityPolicy.argtypes = [c_void_p, c_char_p]
        lib.MagickSetSecurityPolicy.restype = c_bool
    except AttributeError:
        lib.MagickSetSecurityPolicy = None
    lib.MagickSetSize.argtypes = [c_void_p, c_uint, c_uint]
    lib.MagickSetSize.restype = c_bool
    lib.MagickSetSizeOffset.argtypes = [
        c_void_p, c_size_t, c_size_t, c_ssize_t
    ]
    lib.MagickSetSizeOffset.restype = c_bool
    lib.MagickSetType.argtypes = [c_void_p, c_int]
    lib.MagickSetType.restype = c_bool



import os
import os.path
try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

from wand.version import VERSION


def readme():
    with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as f:
        return f.read()


try:
    from setuptools.command.test import test
except ImportError:
    cmdclass = {}
else:
    class pytest(test):

        def finalize_options(self):
            test.finalize_options(self)
            self.test_args = []
            self.test_suite = True

        def run_tests(self):
            from pytest import main
            errno = main(self.test_args)
            raise SystemExit(errno)
    cmdclass = {'test': pytest}

test_requires = [
    'pytest >= 5.3.5',
]

doc_requires = [
    'Sphinx >= 2.4.1',
]

setup(
    name='Wand',
    packages=['wand', 'wand.cdefs'],
    version=VERSION,
    description='Ctypes-based simple MagickWand API binding for Python',
    long_description=readme(),
    long_description_content_type='text/x-rst',
    license='MIT License',
    author='Hong Minhee',
    author_email='hongminhee' '@' 'member.fsf.org',
    maintainer='E. McConville',
    maintainer_email='emcconville' '@' 'emcconville.com',
    url='http://wand-py.org/',
    tests_require=test_requires,
    extras_require={'doc': doc_requires, 'test': test_requires},
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Programming Language :: Python :: Implementation :: Stackless',
        'Topic :: Multimedia :: Graphics'
    ],
    keywords='ImageMagick ctypes',
    cmdclass=cmdclass,
    project_urls={
        'Documentation': 'https://docs.wand-py.org',
        'Source': 'https://github.com/emcconville/wand',
        'Tracker': 'https://github.com/emcconville/wand/issues',
    }
)



""":mod:`wand.font` --- Fonts
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

.. versionadded:: 0.3.0

:class:`Font` is an object which takes the :attr:`~Font.path` of font file,
:attr:`~Font.size`, :attr:`~Font.color`, and whether to use
:attr:`~Font.antialias`\\ ing.  If you want to use font by its name rather
than the file path, use TTFQuery_ package.  The font path resolution by its
name is a very complicated problem to achieve.

.. seealso::

   TTFQuery_ --- Find and Extract Information from TTF Files
      TTFQuery builds on the `FontTools-TTX`_ package to allow the Python
      programmer to accomplish a number of tasks:

      - query the system to find installed fonts

      - retrieve metadata about any TTF font file

        - this includes the glyph outlines (shape) of individual code-points,
          which allows for rendering the glyphs in 3D (such as is done in
          OpenGLContext)

      - lookup/find fonts by:

        - abstract family type
        - proper font name

      - build simple metadata registries for run-time font matching

.. _TTFQuery: http://ttfquery.sourceforge.net/
.. _FontTools-TTX: http://sourceforge.net/projects/fonttools/

"""
from . import assertions
from .color import Color
from .compat import string_type, text

__all__ = ('Font',)


class Font(tuple):
    """Font struct which is a subtype of :class:`tuple`.

    :param path: the path of the font file
    :type path: :class:`str`, :class:`basestring`
    :param size: the size of typeface.  0 by default which means *autosized*
    :type size: :class:`numbers.Real`
    :param color: the color of typeface.  black by default
    :type color: :class:`~wand.color.Color`
    :param antialias: whether to use antialiasing.  :const:`True` by default
    :type antialias: :class:`bool`
    :param stroke_color: optional color to outline typeface.
    :type stroke_color: :class:`~wand.color.Color`
    :param stroke_width: optional thickness of typeface outline.
    :type stroke_width: :class:`numbers.Real`

    .. versionchanged:: 0.3.9
       The ``size`` parameter becomes optional.  Its default value is
       0, which means *autosized*.

    .. versionchanged:: 0.5.0
       Added ``stroke_color`` & ``stoke_width`` paramaters.
    """

    def __new__(cls, path, size=0, color=None, antialias=True,
                stroke_color=None, stroke_width=None):
        assertions.assert_string(path=path)
        assertions.assert_real(size=size)
        if color is None:
            color = Color('black')
        elif isinstance(color, string_type):
            color = Color(color)
        assertions.assert_color(color=color)
        if stroke_color:
            if isinstance(stroke_color, string_type):
                stroke_color = Color(stroke_color)
            assertions.assert_color(stroke_color=stroke_color)
        if stroke_width is not None:
            assertions.assert_real(stroke_width=stroke_width)
        path = text(path)
        return tuple.__new__(cls, (path, size, color, bool(antialias),
                                   stroke_color, stroke_width))

    @property
    def path(self):
        """(:class:`basestring`) The path of font file."""
        return self[0]

    @property
    def size(self):
        """(:class:`numbers.Real`) The font size in pixels."""
        return self[1]

    @property
    def color(self):
        """(:class:`wand.color.Color`) The font color."""
        return self[2]

    @property
    def antialias(self):
        """(:class:`bool`) Whether to apply antialiasing (``True``)
        or not (``False``).

        """
        return self[3]

    @property
    def stroke_color(self):
        """(:class:`wand.color.Color`) The stroke color."""
        return self[4]

    @property
    def stroke_width(self):
        """(:class:`numbers.Real`) The width of the stroke line."""
        return self[5]

    def __repr__(self):
        return '{0.__module__}.{0.__name__}({1})'.format(
            type(self),
            tuple.__repr__(self)
        )
